#include "geotraceman.h"
#include "bitmaps/geotraceman.xbm"
#include "bitmaps/tick.xbm"
#include "items.h"
#include "trace_resolve.h"

/*	-	-	-	-	-	-	-	-	*/
/*  GeoTraceMan V1.0 : using a borrowed shell from etherman
 *  which has then been mutilated beyond recoginition (Thanks Mike!)
 *
 */

/*
 * Functions defined in this file:
 */
int     main ();
void    initGraphics (), initWidgets (), initDefaults ();
void    initWidgetdefaults (), initBitmaps (), initGL ();
void	initMonoWidgets();

/*
 * Global widget data
 */
Widget  mainPane;                      /* entire window pane */
Widget  menuBox;                       /* Form containing commands, etc. */
Widget  drawingCanvas, sdrawingCanvas;        /* drawing surface */
Widget  wl_totaldelay, wl_lastdelay, wl_numhosts, wl_tracestatus;
Widget  wl_detail, wl_currentzoom, wl_mapstatus;
Widget  panner, pannerform;
Pixmap  geotraceman_icon;

/*
 * Other global data
 */
char   *prog_name;

/*
 * The application resources
 */
AppResources appResources;

/*
 * Non-widget resources obtained from resource manager
 */
static XtResource resources[] = {
    {"widgets", "Widgets", XtRString, sizeof (String),
    XtOffset (AppResources *, widgets), XtRImmediate, ""},
    {"stringDialogWidgets", "StringDialogWidgets", XtRString, sizeof(String),
      XtOffset(AppResources *, stringDialogWidgets), XtRImmediate, "" },
    {"zoomboxWidgets", "zoomboxWidgets", XtRString, sizeof(String),
      XtOffset(AppResources *, zoomboxWidgets), XtRImmediate, "" },

    {"fileWidgets", "fileWidgets", XtRString, sizeof(String),
      XtOffset (AppResources *, fileWidgets), XtRImmediate, ""},

    {"fontPath", "FontPath", XtRString, sizeof (String),
    XtOffset (AppResources *, fontPath), XtRImmediate, (XtPointer) ""},

    {"visualType", "VisualType", XtRString, sizeof (String),
    XtOffset (AppResources *, visualType), XtRImmediate, (XtPointer) ""},

    {"pageType", "PageType", XtRString, sizeof (String),
    XtOffset (AppResources *, pageType), XtRImmediate, (XtPointer) ""},

    {"colourDump", "ColourDump", XtRString, sizeof (String),
    XtOffset (AppResources *, colourDump), XtRImmediate, (XtPointer) ""},
};

/*
 * Non-widget resources set on command line.
 */
static XrmOptionDescRec options[] = {
    {"-I", ".deviceType", XrmoptionSepArg, (XtPointer) ""},
    {"-SE", ".secondaryEthers", XrmoptionSepArg, (XtPointer) ""},
    {"-mono", ".visualType", XrmoptionNoArg, (XtPointer) "mono"},
    {"-color", ".visualType", XrmoptionNoArg, (XtPointer) "colour"},
    {"-colour", ".visualType", XrmoptionNoArg, (XtPointer) "colour"},
    {"-gray", ".visualType", XrmoptionNoArg, (XtPointer) "grey"},
    {"-grey", ".visualType", XrmoptionNoArg, (XtPointer) "grey"},
};

/*
 * Widget and non-widget resources if the application defaults
 * file can't be found.
 * Generated automatically from GeoTraceMan.ad by "ad2c".
 * Comment out the include line (but not the NULL) if you don't want
 * any resources compiled in.
 */
static String fallbackResources[] = {
#include "GeoTraceMan.ad.h"
    NULL
};

/*	-	-	-	-	-	-	-	-	*/

Drawable
get_drawable()
{
	return(XtWindow(drawingCanvas));
}

static void
initGraphics (argcp, argv)
int    *argcp;
char  **argv;
{

    toplevel = XtVaAppInitialize (&appContext, "GeoTraceMan",
	options, XtNumber (options),
	argcp, argv, fallbackResources, NULL, ZERO);
    initConverters (appContext);
    XawSimpleMenuAddGlobalActions (appContext);
    add_actions(appContext);
    add_canvas_actions(appContext);
    XtGetApplicationResources (toplevel,
	(XtPointer) & appResources,
	resources,
	XtNumber (resources),
	NULL, ZERO);

    display = XtDisplay (toplevel);
    screen = XtScreen (toplevel);
    root = RootWindowOfScreen (screen);

    if ((strcmp (appResources.visualType, "color") == 0)
	|| (strcmp (appResources.visualType, "colour") == 0)) {
	nm_display_type = COLOUR;
	printf ("Colour display\n");
    }
    else if (strcmp (appResources.visualType, "mono") == 0) {
	nm_display_type = MONO;
	printf ("Monochrome display\n");
    }
    else if ((strcmp (appResources.visualType, "gray") == 0)
	|| (strcmp (appResources.visualType, "grey") == 0)) {
	nm_display_type = GREY;
	printf ("Grayscale display\n");
    }
    else {
	/* autodetect */
	switch (DefaultDepth (display, 0)) {
	    case 8:
	    case 24:
		printf ("Colour display detected\n");
		nm_display_type = COLOUR;
		break;
	    case 1:
		printf ("Monochrome display detected\n");
		nm_display_type = MONO;
		break;
	    default:
		printf ("Grayscale display detected\n");
		nm_display_type = GREY;
		break;
	}
    }

}

/*
 * Initialises the graphical contexts used by the application
 *
 */
static void
initGCs(w)
	Widget w;
{
	XGCValues values;
	unsigned long valuemask;

	values.plane_mask = 1;
	values.foreground = WhitePixelOfScreen(screen);
	values.background = BlackPixelOfScreen(screen);

	gc_inverse = XtGetGC(w, GCForeground | GCBackground | GCPlaneMask, &values);
}

/*
 * initWidgets: Initialize the widgets given in the .widgets resource,
 *	check for required widgets, and set globals vars.
 */
static void
initWidgets ()
{
    initWidgetsFromString (appResources.widgets, ".widgets", (Widget) NULL);
    if ((pannerform = XtNameToWidget (toplevel, "*pannerForm")) == NULL)
	fail0 ("didn't create widget \"pannerform\"");
    panner = XtVaCreateManagedWidget(
                "panner",                       /* arbitrary widget name */
                pannerWidgetClass,      /* widget class from Label.h */
                pannerform,                       /* parent widget */
        NULL);              /* terminate varargs list */

    if ((mainPane = XtNameToWidget (toplevel, "*mainPane")) == NULL)
	fail0 ("didn't create widget \"mainPane\"");
    if ((menuBox = XtNameToWidget (toplevel, "*menuBox")) == NULL)
	fail0 ("didn't create widget \"menuBox\"");
    if ((drawingCanvas = XtNameToWidget (toplevel, "*drawingCanvas")) == NULL)
	fail0 ("didn't create widget \"drawingCanvas\"");
    /*if ((sdrawingCanvas = XtNameToWidget (toplevel, "*worldXBM")) == NULL)*/
	/*fail0 ("didn't create widget \"sdrawingCanvas\"");*/

    if ((wl_totaldelay = XtNameToWidget (toplevel, "*totaltimenLabel")) == NULL)
	fail0 ("didn't create widget \"wl_totaldelay\"");
    if ((wl_lastdelay = XtNameToWidget (toplevel, "*lasthostnLabel")) == NULL)
	fail0 ("didn't create widget \"wl_lastdelay\"");
    if ((wl_numhosts = XtNameToWidget (toplevel, "*numhostsnLabel")) == NULL)
	fail0 ("didn't create widget \"wl_numhosts\"");
    if ((wl_tracestatus = XtNameToWidget (toplevel, "*tstatusnLabel")) == NULL)
	fail0 ("didn't create widget \"wl_tracestatus\"");
    if ((wl_detail = XtNameToWidget (toplevel, "*zoomnLabel")) == NULL)
	fail0 ("didn't create widget \"wl_detail\"");
    if ((wl_currentzoom = XtNameToWidget (toplevel, "*currentnzoomLabel")) == NULL)
	fail0 ("didn't create widget \"wl_currentzoom\"");
    if ((wl_mapstatus = XtNameToWidget (toplevel, "*mstatusnLabel")) == NULL)
	fail0 ("didn't create widget \"wl_mapstatus\"");

    /* if the screen type is monochrome, initialise the widgets accordingly */
    if (nm_display_type == MONO)
	initMonoWidgets();
}

/*
 *  Initialises the colour widgets already created to monochrome.
 */

void
initMonoWidgets()
{
    Widget tmpWidget;

    if ((tmpWidget = XtNameToWidget(toplevel, "*quitButton")) == NULL)
        fail0("No such widget *quitButton");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*menuBox")) == NULL)
	fail0("No such widget *menuBox");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_WHITE, XtNforeground, MONO_BLACK, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*bigForm")) == NULL)
	fail0("No such widget *bigForm");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_WHITE, XtNforeground, MONO_BLACK, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*mapmenuButton")) == NULL)
        fail0("No such widget *mapmenuButton");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*includemenuButton")) == NULL)
        fail0("No such widget *includemenuButton");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*labelmenuButton")) == NULL)
        fail0("No such widget *labelmenuButton");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*filemenuButton")) == NULL)
        fail0("No such widget *filemenuButton");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*miscmenuButton")) == NULL)
        fail0("No such widget *miscmenuButton");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*starttraceButton")) == NULL)
        fail0("No such widget *starttraceButton");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*killtraceButton")) == NULL)
        fail0("No such widget *killtraceButton");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*removetraceButton")) == NULL)
	fail0("No such widget *removetraceButton");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*zoomButton")) == NULL)
        fail0("No such widget *zoomButton");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*cancelzoomButton")) == NULL)
        fail0("No such widget *cancelzoomButton");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*resetzoomButton")) == NULL)
        fail0("No such widget *resetzoomButton");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*tstatsLabel")) == NULL)
        fail0("No such widget *tstatsLabel");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*totaltimeLabel")) == NULL)
        fail0("No such widget *totaltimeLabel");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*totaltimenLabel")) == NULL)
        fail0("No such widget *totaltimenLabel");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*lasthostLabel")) == NULL)
        fail0("No such widget *lasthostLabel");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*lasthostnLabel")) == NULL)
        fail0("No such widget *lasthostnLabel");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*numhostsLabel")) == NULL)
        fail0("No such widget *numhostsLabel");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*numhostsnLabel")) == NULL)
        fail0("No such widget *numhostsnLabel");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*tstatusLabel")) == NULL)
        fail0("No such widget *tstatusLabel");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*tstatusnLabel")) == NULL)
        fail0("No such widget *tstatusnLabel");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*mstatsLabel")) == NULL)
        fail0("No such widget *mstatsLabel");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*zoomLabel")) == NULL)
        fail0("No such widget *zoomLabel");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*zoomnLabel")) == NULL)
        fail0("No such widget *zoomnLabel");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*currentzoomLabel")) == NULL)
        fail0("No such widget *currentzoomLabel");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*currentnzoomLabel")) == NULL)
        fail0("No such widget *currentnzoomLabel");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*mstatusLabel")) == NULL)
        fail0("No such widget *mstatusLabel");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*mstatusnLabel")) == NULL)
        fail0("No such widget *mstatusnLabel");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*drawingCanvas")) == NULL)
        fail0("No such widget *drawingCanvas");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*mapLabel")) == NULL)
        fail0("No such widget *mapLabel");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*traceLabel")) == NULL)
        fail0("No such widget *traceLabel");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
    if ((tmpWidget = XtNameToWidget(toplevel, "*controtButton")) == NULL)
        fail0("No such widget *controtButton");
    XtVaSetValues(tmpWidget, XtNbackground, MONO_BLACK, XtNforeground, MONO_WHITE, NULL);
}

/*	-	-	-	-	-	-	-	-	*/

static void
initDefaults ()
{
    
    nm_postscript_output_file = "geo_postscript_dump_file.ps";

if (!batchmode)
{
    /* get ps page type */
    if (strcmp (appResources.pageType, "") == 0) {
	/* default A4 */
	nm_page_type = A4;
	printf ("Defaulting to A4 size postscript dumps\n");
    }
    else if (strcmp (appResources.pageType, "A4") == 0) {
	nm_page_type = A4;
	printf ("Page type: A4\n");
    }
    else if (strcmp (appResources.pageType, "US_letter") == 0) {
	nm_page_type = US_LETTER;
	printf ("Page type: US_letter\n");
    }

    /* Produce colour ps dumps ? */
    nm_colour_dump = False;	       /* default false */
    if (strcmp (appResources.colourDump, "False") == 0) {
	nm_colour_dump = False;
    }
    else if (strcmp (appResources.colourDump, "True") == 0) {
	nm_colour_dump = True;
    }
}
else
{
        nm_page_type = A4;
        nm_colour_dump = True;
}
}

void
initWidgetdefaults ()
{
	/*XtAddCallback (nodeScale, XtNjumpProc, nodeScale_thumbed, (XtPointer) nodeScale_l);*/
	/*XtAddCallback (linkScale, XtNjumpProc, linkScale_thumbed, (XtPointer) linkScale_l);*/
 
}


void
passpixmap( pix_map)
	Pixmap *pix_map;
{
	pix_map = &geotraceman_icon;
}

void
initBitmaps ()
{
	geotraceman_icon = XCreateBitmapFromData (XtDisplay (toplevel),
	RootWindowOfScreen (XtScreen (toplevel)),
	geotraceman_icon_bits, geotraceman_icon_width, geotraceman_icon_height);

	tick_pixmap = XCreateBitmapFromData (XtDisplay (toplevel),
        RootWindowOfScreen (XtScreen (toplevel)),
        opentick_bits, opentick_width, opentick_height);
}

/*
Do the initial setup for the map canvas
*/
void
initGL ()
{
	float fontx, fonty;

	vinit ("X11");
	vo_xt_window (XtDisplay (drawingCanvas), XtWindow (drawingCanvas), WINDOWX, WINDOWY);
	screen_width = WINDOWX;
	screen_height = WINDOWY;

	vsetflush (0);
	ginit ();
	gconfig();

	doublebuffer ();

	update_display();

#ifdef HERSHEY
	if (strcmp (appResources.fontPath, "") != 0) {
		/*hsetpath( appResources.fontPath); */
	}
    
	hfont ("futura.l");
#endif

#ifdef VOGLFONT
	font ((short) 0);		       /* vogl font */
#endif

#ifdef HERSHEY
	font_size_x(&fontx);
	font_size_y(&fonty);
	htextsize(fontx, fonty);
#endif

}

/*
Initialise the map state structure variables, and read in the initial map 
*/
void
init_map_state()
{

	/* Get the initial level of Map Detail (Should be set depending on machine speed) */
	ms.detail = Initial_Map_Detail;

	/* read in maps */
	read_cil_map();
	read_other_map("riv");
	read_other_map("bdy");

	/* Initialise the offsets and zoom  */	
	ms.x_reg_offset = 0;
	ms.x_offset = 0;
	ms.zoom = 1.0;
	ms.rotation_fraction = Map_Rotation_Fraction;

	/* Initialise the x and y clipping boundaries to the extremes */
	ms.x.min_vis = 0;
	ms.x.max_vis = ms.max_range - 1;
	ms.x.min_vis_reg = 0;
	ms.x.max_vis_reg = ms.side_size - 1;

	ms.y.min_vis = 0;
	ms.y.max_vis = ms.max_range - 1;
	ms.y.min_vis_reg = 0;
	ms.y.max_vis_reg = ms.side_size - 1;

if (!batchmode)
{
	/* Include options set to False */
	ms.includes = 0;
	ms.includes = ~INCL_CIT & ~INCL_RIV & ~INCL_PDY & ~INCL_3D;
}

	/* intialise the rotation direction to rotate to the right */
	ms.rot_dir = '?';
	ms.rotating = False;
	ms.rotation = 0;

if (!batchmode)
{
	/* Set bitmaps to nothing */
	unsetTick(ITEM_INCLUDE_RIVS);
	unsetTick(ITEM_INCLUDE_PBYS);
	unsetTick(ITEM_INCLUDE_CITS);		
	unsetTick(ITEM_INCLUDE_3D);		
}

}

/*
 * Mainline
 *
 */

int
main (argc, argv)
	int     argc;
	char  **argv;
{
	char    buf[32];	/* Holds the name of the application */
	char    trace_to[80];	/* Craig */
        Boolean not_done;

        Trace_Struct *tracep;
        struct hostent *host_ent;
        unsigned long host_addr;
	int craig;
int timeout=0;
long rotation; 



	/* Trace Variables */
	int tracepid;		/* Process Id of the currently running trace */
	int ipnumfh;		/* File handle of the pipe connecting the two */
	int numtrace = 0;	/* Number of traces stored */

	/* God knows what this does, see mike later and fix it */
	prog_name = argv[0];

batchmode=False;

if (argc > 1) 
   if ((argv[1][0] == '-')  && (argv[1][1] == 'T'))
	batchmode=True;

/* Need a batchmode option for 3d Globe centered on Australia (-H)
   3D Globe Centered on USA (-O)
   3D globe Centered on Europe (-M)
   3D globe Centered on Pacific (-A)
*/

if (argc > 1) 
   if ((argv[1][0] == '-')  && (argv[1][1] == 'H'))
   {
	batchmode=True;
	ms.includes=INCL_3D;
   }
if (argc > 1) 
   if ((argv[1][0] == '-')  && (argv[1][1] == 'O'))
   {
/* globe centered on the USA */
	batchmode=True;
	ms.includes=INCL_3D;
   }
if (argc > 1) 
   if ((argv[1][0] == '-')  && (argv[1][1] == 'M'))
   {
/* globe centered on Europe */
	batchmode=True;
	ms.includes=INCL_3D;
   }
if (argc > 1) 
   if ((argv[1][0] == '-')  && (argv[1][1] == 'A'))
   {
	batchmode=True;
	ms.includes=INCL_3D;
   }
if (argc > 1) 
   if ((argv[1][0] == '-')  && (argv[1][1] == 'S'))
   {
	batchmode=True;
   }

if (!batchmode)
{
	initGraphics (&argc, argv);
	initGCs(toplevel);
}
else
	strcpy(trace_to, argv[2]);

	initDefaults ();  

if (argc == 4)
{
    nm_postscript_output_file=argv[3];
}

if (!batchmode)
{
	initWidgets ();
	initWidgetdefaults ();
	initBitmaps ();
	XtRealizeWidget (toplevel);
}

	/* Zero the trace state structure */
	bzero(ts,sizeof(Traces_Status_Struct));

	table_loadhosts(Backbone_Host_Cache);
	table_loadhosts(Local_Host_Cache);
	table_loadnets(Networks_Cache);

	/* Initialise the ms variables to reflect the starting state */
	init_map_state();

	/* Initialise the canvas */
if (!batchmode)
{
	initGL();
	/* Set window title to indicate version */
	sprintf (buf, "Geographical Trace-Route V1.0 Copyright (c) 1993 Curtin University ");
	XStoreName (display, XtWindow (toplevel), buf);

	/* set icon */
	XtVaSetValues (toplevel, XtNiconPixmap, geotraceman_icon, NULL);
	XFlush (XtDisplay (toplevel));

	/* Enter the application main loop, and never return */
	XtAppMainLoop (appContext);	   
}
else
{
	printf("we are in batchmode tracing to %s\n", &trace_to[0]); 
        /* check if host is entered by name or number */
        if (isdigit(*trace_to) == 0) {
                host_ent = gethostbyname(trace_to);
        }
        else {
                host_addr = inet_addr(trace_to);
                host_ent = gethostbyaddr(&host_addr, 4, AF_INET);
        }

        if (host_ent == NULL) {
                printf("Can't contact host\n");
                return;
        }

        if ((ts.tracepid = fork_trace(host_ent->h_name, &ts.tracesd)) != 0) {
                ts.tracing = True;
                if (ts.numtrace == 0)
                        create_trace(&(ts.traces));
                else {
                        go_to_trace((ts.numtrace -1), &tracep);
                        create_trace(&tracep->next_trace);
                }
	not_done=False;
	while(!not_done)
	{
                not_done = readtracepipeanddraw(ts.tracesd);
sleep(1);
timeout++;
if(timeout > 2000) /* cant trace for more than 20 minutes max */
{
printf("this trace has failed bailing out\n");
exit(0);
}
/*craig=getchar(); */
	}

if (argc > 1)
   if ((argv[1][0] == '-')  && (argv[1][1] == 'T'))
      rotation = (long) (((float) ms.max_range) * (5 * ms.rotation_fraction));
if (argc > 1)
   if ((argv[1][0] == '-')  && (argv[1][1] == 'H'))
      rotation = (long) (((float) ms.max_range) * (6 * ms.rotation_fraction));
if (argc > 1)
   if ((argv[1][0] == '-')  && (argv[1][1] == 'O'))
      rotation = (long) (((float) ms.max_range) * (2 * ms.rotation_fraction));
if (argc > 1)
   if ((argv[1][0] == '-')  && (argv[1][1] == 'A'))
      rotation = (long) (((float) ms.max_range) * (4 * ms.rotation_fraction));



if (argc > 1)
   if ((argv[1][0] == '-')  && (argv[1][1] != 'T'))
   {
update_x_offset(rotation);
update_region_delimiters();
   }


        page_flag = 0;
        page_begin = 1;

        ps_color = 3;
        ps_dump_regions(cil_map);

        cities_display_ps();
        display_traces_ps();
        page_flag = 1;
        ps_dump_regions(cil_map);


        }

}
}
